package javamag.asm;

import org.objectweb.asm.ClassReader;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.util.CheckClassAdapter;
import org.objectweb.asm.util.TraceClassVisitor;

import java.io.InputStream;
import java.io.PrintWriter;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public final class OfflineUnsynchronizer {

    public static void main(String[] args) {
        OfflineUnsynchronizer offline = new OfflineUnsynchronizer();
        String fName = null;
        if (args.length > 0) {
            fName = args[0];
        } else {
             fName = "javamag/asm/Counter.class";
        }
        offline.run(fName);
    }

    public static Pattern CLASSFILE_NAME = Pattern.compile("(.*)\\.class");

    private void run(String fName) {
        try (InputStream in = Files.newInputStream(Paths.get(fName))) {
            ClassReader classReader = new ClassReader(in);
            ClassWriter writer = new ClassWriter(classReader, ClassWriter.COMPUTE_FRAMES);

            ClassVisitor unsynchronizer = new UnsynchronizingClassVisitor(writer);
            classReader.accept(unsynchronizer, ClassReader.SKIP_FRAMES | ClassReader.SKIP_DEBUG);

            Path newClazz = Paths.get(transformName(fName));
            Files.write(newClazz, writer.toByteArray());
        } catch (Exception ex) {
            System.err.println("Exception whilst reading class: " + fName);
            ex.printStackTrace(System.err);
        }
    }

    public static String transformName(String fName) {
        Matcher m = CLASSFILE_NAME.matcher(fName);
        if (m.find()) {
            return "test/" + m.group(1) + ".class";
        }
        throw new IllegalStateException(fName + " is not a legal filename");
    }

}
